@@ -590,19 +590,24 @@ get_shaper_font_cache (PangoFontset *fontset)
 {
   ShaperFontCache *cache;
 
-  static GQuark cache_quark = 0;
+  static GQuark cache_quark = 0; /* MT-safe */
   if (G_UNLIKELY (!cache_quark))
     cache_quark = g_quark_from_static_string ("pango-shaper-font-cache");
 
+retry:
   cache = g_object_get_qdata (G_OBJECT (fontset), cache_quark);
-  if (!cache)
+  if (G_UNLIKELY (!cache))
     {
       cache = g_slice_new (ShaperFontCache);
       cache->hash = g_hash_table_new_full (g_direct_hash, NULL,
 					   NULL, (GDestroyNotify)shaper_font_element_destroy);
-
-      g_object_set_qdata_full (G_OBJECT (fontset), cache_quark,
-			       cache, (GDestroyNotify)shaper_font_cache_destroy);
+      if (!g_object_replace_qdata (G_OBJECT (fontset), cache_quark, NULL,
+                                   cache, (GDestroyNotify)shaper_font_cache_destroy,
+                                   NULL))
+        {
+          shaper_font_cache_destroy (cache);
+          goto retry;
+        }
     }
 
   return cache;
@@ -1275,14 +1280,13 @@ compute_derived_language (PangoLanguage *lang,
 static PangoMap *
 get_lang_map (PangoLanguage *lang)
 {
-  static guint engine_type_id = 0;
-  static guint render_type_id = 0;
+  static guint engine_type_id = 0; /* MT-safe */
+  static guint render_type_id = 0; /* MT-safe */
 
   if (engine_type_id == 0)
-    {
-      engine_type_id = g_quark_from_static_string (PANGO_ENGINE_TYPE_LANG);
-      render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_NONE);
-    }
+    engine_type_id = g_quark_from_static_string (PANGO_ENGINE_TYPE_LANG);
+  if (render_type_id == 0)
+    render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_NONE);
 
   return pango_find_map (lang, engine_type_id, render_type_id);
 }
@@ -1366,10 +1370,10 @@ itemize_state_update_for_new_run (ItemizeState *state)
 static const char *
 string_from_script (PangoScript script)
 {
-  static GEnumClass *class = NULL;
+  static GEnumClass *class = NULL; /* MT-safe */
   GEnumValue *value;
-  if (!class)
-    class = g_type_class_ref (PANGO_TYPE_SCRIPT);
+  if (g_once_init_enter ((gsize*)&class))
+    g_once_init_leave((gsize*)&class, (gsize)g_type_class_ref (PANGO_TYPE_SCRIPT));
 
   value = g_enum_get_value (class, script);
   if (!value)
