@@ -180,7 +180,7 @@ fallback_engine_covers (PangoEngineShape *engine G_GNUC_UNUSED,
 
 static GType pango_fallback_engine_get_type (void);
 
-G_DEFINE_ABSTRACT_TYPE (PangoFallbackEngine, pango_fallback_engine, PANGO_TYPE_ENGINE_SHAPE);
+G_DEFINE_TYPE (PangoFallbackEngine, pango_fallback_engine, PANGO_TYPE_ENGINE_SHAPE);
 
 static void
 pango_fallback_engine_init (PangoFallbackEngine *self)
@@ -197,9 +197,9 @@ pango_fallback_engine_class_init (PangoFallbackEngineClass *class)
 PangoEngineShape *
 _pango_get_fallback_shaper (void)
 {
-  static PangoEngineShape *fallback_shaper = NULL;
-  if (!fallback_shaper)
-    fallback_shaper = g_object_new (pango_fallback_engine_get_type (), NULL);
+  static PangoEngineShape *fallback_shaper = NULL; /* MT-safe */
+  if (g_once_init_enter ((gsize*)&fallback_shaper))
+    g_once_init_leave((gsize*)&fallback_shaper, (gsize)g_object_new (pango_fallback_engine_get_type (), NULL));
 
   return fallback_shaper;
 }
