@@ -28,198 +28,6 @@
 #include "pango-font.h"
 #include "pango-enum-types.h"
 #include "pango-impl-utils.h"
-#include "pango-utils-internal.h"
-
-/**
- * SECTION:markup
- * @title:Markup
- * @short_description:Simple markup language for text with attributes
- *
- * Frequently, you want to display some text to the user with attributes
- * applied to part of the text (for example, you might want bold or
- * italicized words). With the base Pango interfaces, you could create a
- * #PangoAttrList and apply it to the text; the problem is that you'd
- * need to apply attributes to some numeric range of characters, for
- * example "characters 12-17." This is broken from an internationalization
- * standpoint; once the text is translated, the word you wanted to
- * italicize could be in a different position.
- *
- * The solution is to include the text attributes in the string to be
- * translated. Pango provides this feature with a small markup language.
- * You can parse a marked-up string into the string text plus a
- * #PangoAttrList using either of pango_parse_markup() or
- * pango_markup_parser_new().
- *
- * A simple example of a marked-up string might be:
- * |[
- * <span foreground="blue" size="x-large">Blue text</span> is <i>cool</i>!
- * ]|
- *
- * Pango uses #GMarkup to parse this language, which means that XML
- * features such as numeric character entities such as `&#169;` for
- * © can be used too.
- *
- * The root tag of a marked-up document is `<markup>`, but
- * pango_parse_markup() allows you to omit this tag, so you will most
- * likely never need to use it. The most general markup tag is `<span>`,
- * then there are some convenience tags.
- *
- * ## Span attributes
- *
- * `<span>` has the following attributes:
- *
- * * `font_desc`:
- *   A font description string, such as "Sans Italic 12".
- *   See pango_font_description_from_string() for a description of the
- *   format of the string representation . Note that any other span
- *   attributes will override this description. So if you have "Sans Italic"
- *   and also a `style="normal"` attribute, you will get Sans normal,
- *   not italic.
- *
- * * `font_family`:
- *   A font family name
- *
- * * `font_size`, `size`:
- *   Font size in 1024ths of a point, or one of the absolute
- *   sizes `xx-small`, `x-small`, `small`, `medium`, `large`,
- *   `x-large`, `xx-large`, or one of the relative sizes `smaller`
- *   or `larger`. If you want to specify a absolute size, it's usually
- *   easier to take advantage of the ability to specify a partial
- *   font description using `font`; you can use `font='12.5'`
- *   rather than `size='12800'`.
- *
- * * `font_style`:
- *   One of `normal`, `oblique`, `italic`
- *
- * * `font_weight`:
- *   One of `ultralight`, `light`, `normal`, `bold`,
- *   `ultrabold`, `heavy`, or a numeric weight
- *
- * * `font_variant`:
- *   One of `normal` or `smallcaps`
- *
- * * `font_stretch`, `stretch`:
- *   One of `ultracondensed`, `extracondensed`, `condensed`,
- *   `semicondensed`, `normal`, `semiexpanded`, `expanded`,
- *   `extraexpanded`, `ultraexpanded`
- *
- * * `font_features`:
- *   A comma-separated list of OpenType font feature
- *   settings, in the same syntax as accepted by CSS. E.g:
- *   `font_features='dlig=1, -kern, afrc on'`
- *
- * * `foreground`, `fgcolor`:
- *   An RGB color specification such as `#00FF00` or a color
- *   name such as `red`. Since 1.38, an RGBA color specification such
- *   as `#00FF007F` will be interpreted as specifying both a foreground
- *   color and foreground alpha.
- *
- * * `background`, `bgcolor`:
- *   An RGB color specification such as `#00FF00` or a color
- *   name such as `red`.
- *   Since 1.38, an RGBA color specification such as `#00FF007F` will
- *   be interpreted as specifying both a background color and
- *   background alpha.
- *
- * * `alpha`, `fgalpha`:
- *   An alpha value for the foreground color, either a plain
- *   integer between 1 and 65536 or a percentage value like `50%`.
- *
- * * `background_alpha`, `bgalpha`:
- *   An alpha value for the background color, either a plain
- *   integer between 1 and 65536 or a percentage value like `50%`.
- *
- * * `underline`:
- *   One of `none`, `single`, `double`, `low`, `error`,
- *   `single-line`, `double-line` or `error-line`.
- *
- * * `underline_color`:
- *   The color of underlines; an RGB color
- *   specification such as `#00FF00` or a color name such as `red`
- *
- * * `overline`:
- *   One of `none` or `single`
- *
- * * `overline_color`:
- *   The color of overlines; an RGB color
- *   specification such as `#00FF00` or a color name such as `red`
- *
- * * `rise`:
- *   Vertical displacement, in Pango units. Can be negative for
- *   subscript, positive for superscript.
- *
- * * `strikethrough`
- *   `true` or `false` whether to strike through the text
- *
- * * `strikethrough_color`:
- *   The color of strikethrough lines; an RGB
- *   color specification such as `#00FF00` or a color name such as `red`
- *
- * * `fallback`:
- *   `true` or `false` whether to enable fallback. If
- *   disabled, then characters will only be used from the closest
- *   matching font on the system. No fallback will be done to other
- *   fonts on the system that might contain the characters in the text.
- *   Fallback is enabled by default. Most applications should not
- *   disable fallback.
- *
- * * `allow_breaks`:
- *   `true` or `false` whether to allow line breaks or not. If
- *   not allowed, the range will be kept in a single run as far
- *   as possible. Breaks are allowed by default.
- *
- * * `insert_hyphens`:`
- *   `true` or `false` whether to insert hyphens when breaking
- *   lines in the middle of a word. Hyphens are inserted by default.
- *
- * * `show`:
- *   A value determining how invisible characters are treated.
- *   Possible values are `spaces`, `line-breaks`, `ignorables`
- *   or combinations, such as `spaces|line-breaks`.
- *
- * * `lang`:
- *   A language code, indicating the text language
- *
- * * `letter_spacing`:
- *   Inter-letter spacing in 1024ths of a point.
- *
- * * `gravity`:
- *   One of `south`, `east`, `north`, `west`, `auto`.
- *
- * * `gravity_hint`:
- *   One of `natural`, `strong`, `line`.
- *
- * ## Convenience tags
- *
- * The following convenience tags are provided:
- *
- * * `<b>`:
- *   Bold
- *
- * * `<big>`:
- *   Makes font relatively larger, equivalent to `<span size="larger">`
- *
- * * `<i>`:
- *   Italic
- *
- * * `<s>`:
- *   Strikethrough
- *
- * * `<sub>`:
- *   Subscript
- *
- * * `<sup>`:
- *   Superscript
- *
- * * `<small>`:
- *   Makes font relatively smaller, equivalent to `<span size="smaller">`
- *
- * * `<tt>`:
- *   Monospace
- *
- * * `<u>`:
- *   Underline
- */
 
 /* FIXME */
 #define _(x) x
@@ -739,65 +547,6 @@ static const GMarkupParser pango_markup_parser = {
   NULL
 };
 
-static void
-destroy_markup_data (MarkupData *md)
-{
-  g_slist_free_full (md->tag_stack, (GDestroyNotify) open_tag_free);
-  g_slist_free_full (md->to_apply, (GDestroyNotify) pango_attribute_destroy);
-  if (md->text)
-      g_string_free (md->text, TRUE);
-
-  if (md->attr_list)
-    pango_attr_list_unref (md->attr_list);
-
-  g_slice_free (MarkupData, md);
-}
-
-static GMarkupParseContext *
-pango_markup_parser_new_internal (char       accel_marker,
-				  GError   **error,
-				  gboolean   want_attr_list)
-{
-  MarkupData *md;
-  GMarkupParseContext *context;
-
-  md = g_slice_new (MarkupData);
-
-  /* Don't bother creating these if they weren't requested;
-   * might be useful e.g. if you just want to validate
-   * some markup.
-   */
-  if (want_attr_list)
-    md->attr_list = pango_attr_list_new ();
-  else
-    md->attr_list = NULL;
-
-  md->text = g_string_new (NULL);
-
-  md->accel_marker = accel_marker;
-  md->accel_char = 0;
-
-  md->index = 0;
-  md->tag_stack = NULL;
-  md->to_apply = NULL;
-
-  context = g_markup_parse_context_new (&pango_markup_parser,
-					0, md,
-                                        (GDestroyNotify)destroy_markup_data);
-
-  if (!g_markup_parse_context_parse (context,
-                                     "<markup>",
-                                     -1,
-                                     error))
-    goto error;
-
-  return context;
-
- error:
-  g_markup_parse_context_free (context);
-  return NULL;
-}
-
 /**
  * pango_parse_markup:
  * @markup_text: markup to parse (see <link linkend="PangoMarkupFormat">markup format</link>)
@@ -820,8 +569,6 @@ pango_markup_parser_new_internal (char       accel_marker,
  * Two @accel_marker characters following each other produce a single
  * literal @accel_marker character.
  *
- * To parse a stream of pango markup incrementally, use pango_markup_parser_new().
- *
  * If any error happens, none of the output arguments are touched except
  * for @error.
  *
@@ -837,12 +584,40 @@ pango_parse_markup (const char                 *markup_text,
 		    GError                    **error)
 {
   GMarkupParseContext *context = NULL;
-  gboolean ret = FALSE;
+  MarkupData *md = NULL;
+  gboolean needs_root = TRUE;
+  GSList *tmp_list;
   const char *p;
   const char *end;
 
   g_return_val_if_fail (markup_text != NULL, FALSE);
 
+  md = g_slice_new (MarkupData);
+
+  /* Don't bother creating these if they weren't requested;
+   * might be useful e.g. if you just want to validate
+   * some markup.
+   */
+  if (attr_list)
+    md->attr_list = pango_attr_list_new ();
+  else
+    md->attr_list = NULL;
+
+  md->text = g_string_new (NULL);
+
+  if (accel_char)
+    *accel_char = 0;
+
+  md->accel_marker = accel_marker;
+  md->accel_char = 0;
+
+  md->index = 0;
+  md->tag_stack = NULL;
+  md->to_apply = NULL;
+
+  context = g_markup_parse_context_new (&pango_markup_parser,
+					0, md, NULL);
+
   if (length < 0)
     length = strlen (markup_text);
 
@@ -851,112 +626,34 @@ pango_parse_markup (const char                 *markup_text,
   while (p != end && xml_isspace (*p))
     ++p;
 
-  context = pango_markup_parser_new_internal (accel_marker,
-                                              error,
-                                              (attr_list != NULL));
-  if (context == NULL)
-    goto out;
+  if (end - p >= 8 && strncmp (p, "<markup>", 8) == 0)
+     needs_root = FALSE;
+
+  if (needs_root)
+    if (!g_markup_parse_context_parse (context,
+				       "<markup>",
+				       -1,
+				       error))
+      goto error;
+
 
   if (!g_markup_parse_context_parse (context,
-                                     markup_text,
-                                     length,
-                                     error))
-    goto out;
+				     markup_text,
+				     length,
+				     error))
+    goto error;
 
-  if (!pango_markup_parser_finish (context,
-                                   attr_list,
-                                   text,
-                                   accel_char,
-                                   error))
-    goto out;
-
-  ret = TRUE;
-
- out:
-  if (context != NULL)
-    g_markup_parse_context_free (context);
-  return ret;
-}
-
-/**
- * pango_markup_parser_new:
- * @accel_marker: character that precedes an accelerator, or 0 for none
- *
- * Parses marked-up text (see
- * <link linkend="PangoMarkupFormat">markup format</link>) to create
- * a plain-text string and an attribute list.
- *
- * If @accel_marker is nonzero, the given character will mark the
- * character following it as an accelerator. For example, @accel_marker
- * might be an ampersand or underscore. All characters marked
- * as an accelerator will receive a %PANGO_UNDERLINE_LOW attribute,
- * and the first character so marked will be returned in @accel_char,
- * when calling finish(). Two @accel_marker characters following each
- * other produce a single literal @accel_marker character.
- *
- * To feed markup to the parser, use g_markup_parse_context_parse()
- * on the returned #GMarkupParseContext. When done with feeding markup
- * to the parser, use pango_markup_parser_finish() to get the data out
- * of it, and then use g_markup_parse_context_free() to free it.
- *
- * This function is designed for applications that read pango markup
- * from streams. To simply parse a string containing pango markup,
- * the simpler pango_parse_markup() API is recommended instead.
- *
- * Return value: (transfer none): a #GMarkupParseContext that should be
- * destroyed with g_markup_parse_context_free().
- *
- * Since: 1.31.0
- **/
-GMarkupParseContext *
-pango_markup_parser_new (gunichar               accel_marker)
-{
-  GError *error = NULL;
-  GMarkupParseContext *context;
-  context = pango_markup_parser_new_internal (accel_marker, &error, TRUE);
-
-  if (context == NULL)
-    g_critical ("Had error when making markup parser: %s\n", error->message);
-
-  return context;
-}
-
-/**
- * pango_markup_parser_finish:
- * @context: A valid parse context that was returned from pango_markup_parser_new()
- * @attr_list: (out) (allow-none): address of return location for a #PangoAttrList, or %NULL
- * @text: (out) (allow-none): address of return location for text with tags stripped, or %NULL
- * @accel_char: (out) (allow-none): address of return location for accelerator char, or %NULL
- * @error: address of return location for errors, or %NULL
- *
- * After feeding a pango markup parser some data with g_markup_parse_context_parse(),
- * use this function to get the list of pango attributes and text out of the
- * markup. This function will not free @context, use g_markup_parse_context_free()
- * to do so.
- *
- * Return value: %FALSE if @error is set, otherwise %TRUE
- *
- * Since: 1.31.0
- */
-gboolean
-pango_markup_parser_finish (GMarkupParseContext   *context,
-                            PangoAttrList        **attr_list,
-                            char                 **text,
-                            gunichar              *accel_char,
-                            GError               **error)
-{
-  gboolean ret = FALSE;
-  MarkupData *md = g_markup_parse_context_get_user_data (context);
-  GSList *tmp_list;
-
-  if (!g_markup_parse_context_parse (context,
-                                     "</markup>",
-                                     -1,
-                                     error))
-    goto out;
+  if (needs_root)
+    if (!g_markup_parse_context_parse (context,
+				       "</markup>",
+				       -1,
+				       error))
+      goto error;
 
   if (!g_markup_parse_context_end_parse (context, error))
-    goto out;
+    goto error;
+
+  g_markup_parse_context_free (context);
 
   if (md->attr_list)
     {
@@ -978,25 +675,38 @@ pango_markup_parser_finish (GMarkupParseContext   *context,
     }
 
   if (attr_list)
-    {
-      *attr_list = md->attr_list;
-      md->attr_list = NULL;
-    }
+    *attr_list = md->attr_list;
 
   if (text)
-    {
-      *text = g_string_free (md->text, FALSE);
-      md->text = NULL;
-    }
+    *text = g_string_free (md->text, FALSE);
+  else
+    g_string_free (md->text, TRUE);
 
   if (accel_char)
     *accel_char = md->accel_char;
 
   g_assert (md->tag_stack == NULL);
-  ret = TRUE;
 
- out:
-  return ret;
+  g_slice_free (MarkupData, md);
+
+  return TRUE;
+
+ error:
+  g_slist_foreach (md->tag_stack, (GFunc) open_tag_free, NULL);
+  g_slist_free (md->tag_stack);
+  g_slist_foreach (md->to_apply, (GFunc) pango_attribute_destroy, NULL);
+  g_slist_free (md->to_apply);
+  g_string_free (md->text, TRUE);
+
+  if (md->attr_list)
+    pango_attr_list_unref (md->attr_list);
+
+  g_slice_free (MarkupData, md);
+
+  if (context)
+    g_markup_parse_context_free (context);
+
+  return FALSE;
 }
 
 static void
@@ -1144,7 +854,7 @@ span_parse_int (const char *attr_name,
 {
   const char *end = attr_val;
 
-  if (!_pango_scan_int (&end, val) || *end != '\0')
+  if (!pango_scan_int (&end, val) || *end != '\0')
     {
       g_set_error (error,
 		   G_MARKUP_ERROR,
@@ -1195,11 +905,10 @@ static gboolean
 span_parse_color (const char *attr_name,
 		  const char *attr_val,
 		  PangoColor *color,
-                  guint16 *alpha,
 		  int line_number,
 		  GError **error)
 {
-  if (!pango_color_parse_with_alpha (color, alpha, attr_val))
+  if (!pango_color_parse (color, attr_val))
     {
       g_set_error (error,
 		   G_MARKUP_ERROR,
@@ -1214,56 +923,6 @@ span_parse_color (const char *attr_name,
   return TRUE;
 }
 
-static gboolean
-span_parse_alpha (const char  *attr_name,
-                  const char  *attr_val,
-                  guint16     *val,
-                  int          line_number,
-                  GError     **error)
-{
-  const char *end = attr_val;
-  int int_val;
-
-  if (_pango_scan_int (&end, &int_val))
-    {
-      if (*end == '\0' && int_val > 0 && int_val <= 0xffff)
-        {
-          *val = (guint16)int_val;
-          return TRUE;
-        }
-      else if (*end == '%' && int_val > 0 && int_val <= 100)
-        {
-          *val = (guint16)(int_val * 0xffff / 100);
-          return TRUE;
-        }
-      else
-        {
-          g_set_error (error,
-                       G_MARKUP_ERROR,
-                       G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("Value of '%s' attribute on <span> tag "
-                         "on line %d could not be parsed; "
-                         "should be between 0 and 65536 or a "
-                         "percentage, not '%s'"),
-                         attr_name, line_number, attr_val);
-          return FALSE;
-        }
-    }
-  else
-    {
-      g_set_error (error,
-		   G_MARKUP_ERROR,
-		   G_MARKUP_ERROR_INVALID_CONTENT,
-		   _("Value of '%s' attribute on <span> tag "
-		     "on line %d could not be parsed; "
-		     "should be an integer, not '%s'"),
-		   attr_name, line_number, attr_val);
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
 static gboolean
 span_parse_enum (const char *attr_name,
 		 const char *attr_val,
@@ -1274,7 +933,7 @@ span_parse_enum (const char *attr_name,
 {
   char *possible_values = NULL;
 
-  if (!_pango_parse_enum (type, attr_val, val, FALSE, &possible_values))
+  if (!pango_parse_enum (type, attr_val, val, FALSE, &possible_values))
     {
       g_set_error (error,
 		   G_MARKUP_ERROR,
@@ -1290,32 +949,6 @@ span_parse_enum (const char *attr_name,
   return TRUE;
 }
 
-static gboolean
-span_parse_flags (const char  *attr_name,
-                  const char  *attr_val,
-                  GType        type,
-                  int         *val,
-                  int          line_number,
-                  GError     **error)
-{
-  char *possible_values = NULL;
-
-  if (!pango_parse_flags (type, attr_val, val, &possible_values))
-    {
-      g_set_error (error,
-                   G_MARKUP_ERROR,
-                   G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("'%s' is not a valid value for the '%s' "
-                     "attribute on <span> tag, line %d; valid "
-                     "values are %s or combinations with |"),
-                   attr_val, attr_name, line_number, possible_values);
-      g_free (possible_values);
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
 static gboolean
 span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 		     OpenTag               *tag,
@@ -1338,8 +971,6 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
   const char *background = NULL;
   const char *underline = NULL;
   const char *underline_color = NULL;
-  const char *overline = NULL;
-  const char *overline_color = NULL;
   const char *strikethrough = NULL;
   const char *strikethrough_color = NULL;
   const char *rise = NULL;
@@ -1348,12 +979,6 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
   const char *fallback = NULL;
   const char *gravity = NULL;
   const char *gravity_hint = NULL;
-  const char *font_features = NULL;
-  const char *alpha = NULL;
-  const char *background_alpha = NULL;
-  const char *allow_breaks = NULL;
-  const char *insert_hyphens = NULL;
-  const char *show = NULL;
 
   g_markup_parse_context_get_position (context,
 				       &line_number, &char_number);
@@ -1382,19 +1007,6 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
       gboolean found = FALSE;
 
       switch (names[i][0]) {
-      case 'a':
-        CHECK_ATTRIBUTE (allow_breaks);
-        CHECK_ATTRIBUTE (alpha);
-        break;
-      case 'b':
-	CHECK_ATTRIBUTE (background);
-	CHECK_ATTRIBUTE2(background, "bgcolor");
-        CHECK_ATTRIBUTE (background_alpha);
-        CHECK_ATTRIBUTE2(background_alpha, "bgalpha");
-        break;
-      case 'c':
-	CHECK_ATTRIBUTE2(foreground, "color");
-        break;
       case 'f':
 	CHECK_ATTRIBUTE (fallback);
 	CHECK_ATTRIBUTE2(desc, "font");
@@ -1409,13 +1021,9 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	CHECK_ATTRIBUTE2(weight, "font_weight");
 
 	CHECK_ATTRIBUTE (foreground);
-	CHECK_ATTRIBUTE2(foreground, "fgcolor");
-	CHECK_ATTRIBUTE2(alpha, "fgalpha");
-
-	CHECK_ATTRIBUTE (font_features);
+	CHECK_ATTRIBUTE2 (foreground, "fgcolor");
 	break;
       case 's':
-	CHECK_ATTRIBUTE (show);
 	CHECK_ATTRIBUTE (size);
 	CHECK_ATTRIBUTE (stretch);
 	CHECK_ATTRIBUTE (strikethrough);
@@ -1426,22 +1034,18 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	CHECK_ATTRIBUTE (gravity);
 	CHECK_ATTRIBUTE (gravity_hint);
 	break;
-      case 'i':
-        CHECK_ATTRIBUTE (insert_hyphens);
-        break;
       case 'l':
 	CHECK_ATTRIBUTE (lang);
 	CHECK_ATTRIBUTE (letter_spacing);
 	break;
-      case 'o':
-	CHECK_ATTRIBUTE (overline);
-	CHECK_ATTRIBUTE (overline_color);
-	break;
       case 'u':
 	CHECK_ATTRIBUTE (underline);
 	CHECK_ATTRIBUTE (underline_color);
 	break;
       default:
+	CHECK_ATTRIBUTE (background);
+	CHECK_ATTRIBUTE2 (background, "bgcolor");
+	CHECK_ATTRIBUTE2(foreground, "color");
 	CHECK_ATTRIBUTE (rise);
 	CHECK_ATTRIBUTE (variant);
 	CHECK_ATTRIBUTE (weight);
@@ -1488,15 +1092,18 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	  const char *end;
 	  gint n;
 
-	  if ((end = size, !_pango_scan_int (&end, &n)) || *end != '\0' || n < 0)
+/* cap size from the top at an arbitrary 2048 */
+#define MAX_SIZE (2048 * PANGO_SCALE)
+
+	  if ((end = size, !pango_scan_int (&end, &n)) || *end != '\0' || n < 0 || n > MAX_SIZE)
 	    {
 	      g_set_error (error,
 			   G_MARKUP_ERROR,
 			   G_MARKUP_ERROR_INVALID_CONTENT,
 			   _("Value of 'size' attribute on <span> tag on line %d "
-			     "could not be parsed; should be an integer no more than %d,"
-			     " or a string such as 'small', not '%s'"),
-			   line_number, INT_MAX, size);
+			     "could not be parsed; should be an integer less than %d, or a "
+			     "string such as 'small', not '%s'"),
+			   line_number, MAX_SIZE+1, size);
 	      goto error;
 	    }
 
@@ -1616,47 +1223,21 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
   if (G_UNLIKELY (foreground))
     {
       PangoColor color;
-      guint16 alpha;
 
-      if (!span_parse_color ("foreground", foreground, &color, &alpha, line_number, error))
+      if (!span_parse_color ("foreground", foreground, &color, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_foreground_new (color.red, color.green, color.blue));
-      if (alpha != 0xffff)
-        add_attribute (tag, pango_attr_foreground_alpha_new (alpha));
     }
 
   if (G_UNLIKELY (background))
     {
       PangoColor color;
-      guint16 alpha;
 
-      if (!span_parse_color ("background", background, &color, &alpha, line_number, error))
+      if (!span_parse_color ("background", background, &color, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_background_new (color.red, color.green, color.blue));
-      if (alpha != 0xffff)
-        add_attribute (tag, pango_attr_background_alpha_new (alpha));
-    }
-
-  if (G_UNLIKELY (alpha))
-    {
-      guint16 val;
-
-      if (!span_parse_alpha ("alpha", alpha, &val, line_number, error))
-        goto error;
-
-      add_attribute (tag, pango_attr_foreground_alpha_new (val));
-    }
-
-  if (G_UNLIKELY (background_alpha))
-    {
-      guint16 val;
-
-      if (!span_parse_alpha ("background_alpha", background_alpha, &val, line_number, error))
-        goto error;
-
-      add_attribute (tag, pango_attr_background_alpha_new (val));
     }
 
   if (G_UNLIKELY (underline))
@@ -1673,32 +1254,12 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
     {
       PangoColor color;
 
-      if (!span_parse_color ("underline_color", underline_color, &color, NULL, line_number, error))
+      if (!span_parse_color ("underline_color", underline_color, &color, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_underline_color_new (color.red, color.green, color.blue));
     }
 
-  if (G_UNLIKELY (overline))
-    {
-      PangoOverline ol = PANGO_OVERLINE_NONE;
-
-      if (!span_parse_enum ("overline", overline, PANGO_TYPE_OVERLINE, (int*)(void*)&ol, line_number, error))
-	goto error;
-
-      add_attribute (tag, pango_attr_overline_new (ol));
-    }
-
-  if (G_UNLIKELY (overline_color))
-    {
-      PangoColor color;
-
-      if (!span_parse_color ("overline_color", overline_color, &color, NULL, line_number, error))
-	goto error;
-
-      add_attribute (tag, pango_attr_overline_color_new (color.red, color.green, color.blue));
-    }
-
   if (G_UNLIKELY (gravity))
     {
       PangoGravity gr = PANGO_GRAVITY_SOUTH;
@@ -1706,19 +1267,6 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
       if (!span_parse_enum ("gravity", gravity, PANGO_TYPE_GRAVITY, (int*)(void*)&gr, line_number, error))
 	goto error;
 
-      if (gr == PANGO_GRAVITY_AUTO)
-        {
-	  g_set_error (error,
-		       G_MARKUP_ERROR,
-		       G_MARKUP_ERROR_INVALID_CONTENT,
-		       _("'%s' is not a valid value for the 'stretch' "
-			 "attribute on <span> tag, line %d; valid "
-			 "values are for example 'south', 'east', "
-			 "'north', 'west'"),
-		       gravity, line_number);
-	  goto error;
-        }
-
       add_attribute (tag, pango_attr_gravity_new (gr));
     }
 
@@ -1746,7 +1294,7 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
     {
       PangoColor color;
 
-      if (!span_parse_color ("strikethrough_color", strikethrough_color, &color, NULL, line_number, error))
+      if (!span_parse_color ("strikethrough_color", strikethrough_color, &color, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_strikethrough_color_new (color.red, color.green, color.blue));
@@ -1762,16 +1310,6 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
       add_attribute (tag, pango_attr_fallback_new (b));
     }
 
-  if (G_UNLIKELY (show))
-    {
-      PangoShowFlags flags;
-
-      if (!span_parse_flags ("show", show, PANGO_TYPE_SHOW_FLAGS, (int*)(void*)&flags, line_number, error))
-	goto error;
-
-      add_attribute (tag, pango_attr_show_new (flags));
-    }
-
   if (G_UNLIKELY (rise))
     {
       gint n = 0;
@@ -1798,31 +1336,6 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 		     pango_attr_language_new (pango_language_from_string (lang)));
     }
 
-  if (G_UNLIKELY (font_features))
-    {
-      add_attribute (tag, pango_attr_font_features_new (font_features));
-    }
-
-  if (G_UNLIKELY (allow_breaks))
-    {
-      gboolean b = FALSE;
-
-      if (!span_parse_boolean ("allow_breaks", allow_breaks, &b, line_number, error))
-	goto error;
-
-      add_attribute (tag, pango_attr_allow_breaks_new (b));
-    }
-
-  if (G_UNLIKELY (insert_hyphens))
-    {
-      gboolean b = FALSE;
-
-      if (!span_parse_boolean ("insert_hyphens", insert_hyphens, &b, line_number, error))
-	goto error;
-
-      add_attribute (tag, pango_attr_insert_hyphens_new (b));
-    }
-
   return TRUE;
 
  error:
