@@ -32,7 +32,6 @@ typedef struct _PangoCairoContextInfo PangoCairoContextInfo;
 struct _PangoCairoContextInfo
 {
   double dpi;
-  gboolean set_options_explicit;
 
   cairo_font_options_t *set_options;
   cairo_font_options_t *surface_options;
@@ -89,7 +88,7 @@ retry:
   return info;
 }
 
-static void
+static gboolean
 _pango_cairo_update_context (cairo_t      *cr,
 			     PangoContext *context)
 {
@@ -109,12 +108,6 @@ _pango_cairo_update_context (cairo_t      *cr,
   if (!info->surface_options)
     info->surface_options = cairo_font_options_create ();
   cairo_surface_get_font_options (target, info->surface_options);
-  if (!info->set_options_explicit)
-  {
-    if (!info->set_options)
-      info->set_options = cairo_font_options_create ();
-    cairo_get_font_options (cr, info->set_options);
-  }
 
   old_merged_options = info->merged_options;
   info->merged_options = NULL;
@@ -151,8 +144,8 @@ _pango_cairo_update_context (cairo_t      *cr,
 
   pango_context_set_matrix (context, &pango_matrix);
 
-  if (changed)
-    pango_context_changed (context);
+
+  return changed;
 }
 
 /**
@@ -175,7 +168,7 @@ pango_cairo_update_context (cairo_t      *cr,
   g_return_if_fail (cr != NULL);
   g_return_if_fail (PANGO_IS_CONTEXT (context));
 
-  _pango_cairo_update_context (cr, context);
+  (void) _pango_cairo_update_context (cr, context);
 }
 
 /**
@@ -225,8 +218,8 @@ pango_cairo_context_get_resolution (PangoContext *context)
 /**
  * pango_cairo_context_set_font_options:
  * @context: a #PangoContext, from a pangocairo font map
- * @options: (nullable): a #cairo_font_options_t, or %NULL to unset
- *           any previously set options. A copy is made.
+ * @options: a #cairo_font_options_t, or %NULL to unset any previously set
+ *           options. A copy is made.
  *
  * Sets the font options used when rendering text with this context.
  * These options override any options that pango_cairo_update_context()
@@ -242,32 +235,15 @@ pango_cairo_context_set_font_options (PangoContext               *context,
 
   g_return_if_fail (PANGO_IS_CONTEXT (context));
 
-  info = get_context_info (context, TRUE);
+  info  = get_context_info (context, TRUE);
 
-  if (!info->set_options && !options)
-    return;
-
-  if (info->set_options &&
-      options &&
-      cairo_font_options_equal (info->set_options, options))
-    return;
-
-  if (info->set_options || options)
-    pango_context_changed (context);
-
- if (info->set_options)
+  if (info->set_options)
     cairo_font_options_destroy (info->set_options);
 
   if (options)
-  {
     info->set_options = cairo_font_options_copy (options);
-    info->set_options_explicit = TRUE;
-  }
   else
-  {
     info->set_options = NULL;
-    info->set_options_explicit = FALSE;
-  }
 
   if (info->merged_options)
     {
@@ -281,12 +257,12 @@ pango_cairo_context_set_font_options (PangoContext               *context,
  * @context: a #PangoContext, from a pangocairo font map
  *
  * Retrieves any font rendering options previously set with
- * pango_cairo_context_set_font_options(). This function does not report options
+ * pango_cairo_font_map_set_font_options(). This function does not report options
  * that are derived from the target surface by pango_cairo_update_context()
  *
- * Return value: (nullable): the font options previously set on the
- *   context, or %NULL if no options have been set. This value is
- *   owned by the context and must not be modified or freed.
+ * Return value: the font options previously set on the context, or %NULL
+ *   if no options have been set. This value is owned by the context
+ *   and must not be modified or freed.
  *
  * Since: 1.10
  **/
@@ -337,8 +313,8 @@ _pango_cairo_context_get_merged_font_options (PangoContext *context)
 /**
  * pango_cairo_context_set_shape_renderer:
  * @context: a #PangoContext, from a pangocairo font map
- * @func: (nullable): Callback function for rendering attributes of
- *        type %PANGO_ATTR_SHAPE, or %NULL to disable shape rendering.
+ * @func: Callback function for rendering attributes of type
+ * %PANGO_ATTR_SHAPE, or %NULL to disable shape rendering.
  * @data: User data that will be passed to @func.
  * @dnotify: Callback that will be called when the
  *           context is freed to release @data, or %NULL.
@@ -382,9 +358,8 @@ pango_cairo_context_set_shape_renderer (PangoContext                *context,
  * attributes of type %PANGO_ATTR_SHAPE as set by
  * pango_cairo_context_set_shape_renderer(), if any.
  *
- * Return value: (transfer none) (nullable): the shape rendering callback previously
- *   set on the context, or %NULL if no shape rendering callback have
- *   been set.
+ * Return value: the shape rendering callback previously set on the context, or %NULL
+ *   if no shape rendering callback have been set.
  *
  * Since: 1.18
  */
@@ -499,6 +474,7 @@ pango_cairo_update_layout (cairo_t     *cr,
   g_return_if_fail (cr != NULL);
   g_return_if_fail (PANGO_IS_LAYOUT (layout));
 
-  _pango_cairo_update_context (cr, pango_layout_get_context (layout));
+  if (_pango_cairo_update_context (cr, pango_layout_get_context (layout)))
+    pango_layout_context_changed (layout);
 }
 
