@@ -37,18 +37,12 @@
 typedef struct _PangoFT2Family       PangoFT2Family;
 typedef struct _PangoFT2FontMapClass PangoFT2FontMapClass;
 
-/**
- * PangoFT2FontMap:
- *
- * The #PangoFT2FontMap is the #PangoFontMap implementation for FreeType fonts.
- */
 struct _PangoFT2FontMap
 {
   PangoFcFontMap parent_instance;
 
   FT_Library library;
 
-  guint serial;
   double dpi_x;
   double dpi_y;
 
@@ -72,8 +66,6 @@ static PangoFcFont * pango_ft2_font_map_new_font            (PangoFcFontMap
 							     FcPattern            *pattern);
 static double        pango_ft2_font_map_get_resolution      (PangoFcFontMap       *fcfontmap,
 							     PangoContext         *context);
-static guint         pango_ft2_font_map_get_serial          (PangoFontMap         *fontmap);
-static void          pango_ft2_font_map_changed             (PangoFontMap         *fontmap);
 
 static PangoFT2FontMap *pango_ft2_global_fontmap = NULL; /* MT-safe */
 
@@ -83,12 +75,9 @@ static void
 pango_ft2_font_map_class_init (PangoFT2FontMapClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  PangoFontMapClass *fontmap_class = PANGO_FONT_MAP_CLASS (class);
   PangoFcFontMapClass *fcfontmap_class = PANGO_FC_FONT_MAP_CLASS (class);
 
   gobject_class->finalize = pango_ft2_font_map_finalize;
-  fontmap_class->get_serial = pango_ft2_font_map_get_serial;
-  fontmap_class->changed = pango_ft2_font_map_changed;
   fcfontmap_class->default_substitute = _pango_ft2_font_map_default_substitute;
   fcfontmap_class->new_font = pango_ft2_font_map_new_font;
   fcfontmap_class->get_resolution = pango_ft2_font_map_get_resolution;
@@ -97,16 +86,9 @@ pango_ft2_font_map_class_init (PangoFT2FontMapClass *class)
 static void
 pango_ft2_font_map_init (PangoFT2FontMap *fontmap)
 {
-  FT_Error error;
-
-  fontmap->serial = 1;
   fontmap->library = NULL;
   fontmap->dpi_x   = 72.0;
   fontmap->dpi_y   = 72.0;
-
-  error = FT_Init_FreeType (&fontmap->library);
-  if (error != FT_Err_Ok)
-    g_critical ("pango_ft2_font_map_init: Could not initialize freetype");
 }
 
 static void
@@ -142,25 +124,19 @@ pango_ft2_font_map_finalize (GObject *object)
 PangoFontMap *
 pango_ft2_font_map_new (void)
 {
-  return (PangoFontMap *) g_object_new (PANGO_TYPE_FT2_FONT_MAP, NULL);
-}
+  PangoFT2FontMap *ft2fontmap;
+  FT_Error error;
 
-static guint
-pango_ft2_font_map_get_serial (PangoFontMap *fontmap)
-{
-  PangoFT2FontMap *ft2fontmap = PANGO_FT2_FONT_MAP (fontmap);
+  /* Make sure that the type system is initialized */
+  g_type_init ();
 
-  return ft2fontmap->serial;
-}
+  ft2fontmap = g_object_new (PANGO_TYPE_FT2_FONT_MAP, NULL);
 
-static void
-pango_ft2_font_map_changed (PangoFontMap *fontmap)
-{
-  PangoFT2FontMap *ft2fontmap = PANGO_FT2_FONT_MAP (fontmap);
+  error = FT_Init_FreeType (&ft2fontmap->library);
+  if (error != FT_Err_Ok)
+    g_critical ("pango_ft2_font_map_new: Could not initialize freetype");
 
-  ft2fontmap->serial++;
-  if (ft2fontmap->serial == 0)
-    ft2fontmap->serial++;
+  return (PangoFontMap *)ft2fontmap;
 }
 
 /**
@@ -184,10 +160,6 @@ pango_ft2_font_map_set_default_substitute (PangoFT2FontMap        *fontmap,
 					   gpointer                data,
 					   GDestroyNotify          notify)
 {
-  fontmap->serial++;
-  if (fontmap->serial == 0)
-    fontmap->serial++;
-
   if (fontmap->substitute_destroy)
     fontmap->substitute_destroy (fontmap->substitute_data);
 
@@ -200,7 +172,7 @@ pango_ft2_font_map_set_default_substitute (PangoFT2FontMap        *fontmap,
 
 /**
  * pango_ft2_font_map_substitute_changed:
- * @fontmap: a #PangoFT2FontMap
+ * @fontmap: a #PangoFT2Fontmap
  *
  * Call this function any time the results of the
  * default substitution function set with
@@ -213,15 +185,12 @@ pango_ft2_font_map_set_default_substitute (PangoFT2FontMap        *fontmap,
 void
 pango_ft2_font_map_substitute_changed (PangoFT2FontMap *fontmap)
 {
-  fontmap->serial++;
-  if (fontmap->serial == 0)
-    fontmap->serial++;
   pango_fc_font_map_cache_clear (PANGO_FC_FONT_MAP (fontmap));
 }
 
 /**
  * pango_ft2_font_map_set_resolution:
- * @fontmap: a #PangoFT2FontMap
+ * @fontmap: a #PangoFT2Fontmap
  * @dpi_x: dots per inch in the X direction
  * @dpi_y: dots per inch in the Y direction
  *
@@ -243,13 +212,12 @@ pango_ft2_font_map_set_resolution (PangoFT2FontMap *fontmap,
 }
 
 /**
- * pango_ft2_font_map_create_context: (skip)
- * @fontmap: a #PangoFT2FontMap
+ * pango_ft2_font_map_create_context:
+ * @fontmap: a #PangoFT2Fontmap
  *
  * Create a #PangoContext for the given fontmap.
  *
- * Return value: (transfer full): the newly created context; free with
- *     g_object_unref().
+ * Return value: the newly created context; free with g_object_unref().
  *
  * Since: 1.2
  *
@@ -264,19 +232,18 @@ pango_ft2_font_map_create_context (PangoFT2FontMap *fontmap)
 }
 
 /**
- * pango_ft2_get_context: (skip)
+ * pango_ft2_get_context:
  * @dpi_x:  the horizontal DPI of the target device
  * @dpi_y:  the vertical DPI of the target device
  *
  * Retrieves a #PangoContext for the default PangoFT2 fontmap
- * (see pango_ft2_font_map_for_display()) and sets the resolution
+ * (see pango_ft2_fontmap_get_for_display()) and sets the resolution
  * for the default fontmap to @dpi_x by @dpi_y.
  *
- * Return value: (transfer full): the new #PangoContext
+ * Return value: the new #PangoContext
  *
  * Deprecated: 1.22: Use pango_font_map_create_context() instead.
  **/
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 PangoContext *
 pango_ft2_get_context (double dpi_x, double dpi_y)
 {
@@ -287,10 +254,9 @@ pango_ft2_get_context (double dpi_x, double dpi_y)
 
   return pango_font_map_create_context (fontmap);
 }
-G_GNUC_END_IGNORE_DEPRECATIONS
 
 /**
- * pango_ft2_font_map_for_display: (skip)
+ * pango_ft2_font_map_for_display:
  *
  * Returns a #PangoFT2FontMap. This font map is cached and should
  * not be freed. If the font map is no longer needed, it can
@@ -298,13 +264,13 @@ G_GNUC_END_IGNORE_DEPRECATIONS
  * global PangoFT2 fontmap is deprecated; use pango_ft2_font_map_new()
  * instead.
  *
- * Return value: (transfer none): a #PangoFT2FontMap.
+ * Return value: a #PangoFT2FontMap.
  **/
 PangoFontMap *
 pango_ft2_font_map_for_display (void)
 {
-  if (g_once_init_enter (&pango_ft2_global_fontmap))
-    g_once_init_leave (&pango_ft2_global_fontmap, PANGO_FT2_FONT_MAP (pango_ft2_font_map_new ()));
+  if (g_once_init_enter ((gsize*)&pango_ft2_global_fontmap))
+    g_once_init_leave((gsize*)&pango_ft2_global_fontmap, (gsize)pango_ft2_font_map_new ());
 
   return PANGO_FONT_MAP (pango_ft2_global_fontmap);
 }
@@ -339,7 +305,7 @@ _pango_ft2_font_map_get_library (PangoFontMap *fontmap)
 
 /**
  * _pango_ft2_font_map_get_renderer:
- * @fontmap: a #PangoFT2FontMap
+ * @fontmap: a #PangoFT2Fontmap
  *
  * Gets the singleton PangoFT2Renderer for this fontmap.
  *
